import sys

from ctypes import cdll
from pathlib import Path

if sys.version_info < (3, 6):
    raise RuntimeError("Python 3.6 or later is required.")

try:
    from nsys_recipe import nsys_constants
    from nsys_recipe.lib import *

    # Add necessary Nsys modules to path.
    modules = [
        nsys_constants.NSYS_RULE_DIR,
        nsys_constants.NSYS_REPORT_DIR,
        nsys_constants.NSYS_PYTHON_LIB_DIR,
        nsys_constants.NSYS_RECIPE_THIRDPARTY_PATH
    ]
    sys.path = [nsys_path.find_installed_file(module) for module in modules] + sys.path

    # Load the SQLite extension library.
    cdll.LoadLibrary(nsys_path.find_installed_file(nsys_constants.NSYS_SQLITE_LIB))

    # Expert System rules
    from nsys_recipe.cuda_api_sync.cuda_api_sync import CudaApiSync
    from nsys_recipe.cuda_memcpy_async.cuda_memcpy_async import CudaMemcpyAsync
    from nsys_recipe.cuda_memcpy_sync.cuda_memcpy_sync import CudaMemcpySync
    from nsys_recipe.cuda_memset_sync.cuda_memset_sync import CudaMemsetSync
    from nsys_recipe.dx12_mem_ops.dx12_mem_ops import Dx12MemOps
    from nsys_recipe.gpu_gaps.gpu_gaps import GpuGaps
    from nsys_recipe.gpu_time_util.gpu_time_util import GpuTimeUtil

    # Stats reports
    from nsys_recipe.cuda_api_sum.cuda_api_sum import CudaApiSum
    from nsys_recipe.cuda_gpu_kern_sum.cuda_gpu_kern_sum import CudaGpuKernSum
    from nsys_recipe.cuda_gpu_mem_size_sum.cuda_gpu_mem_size_sum import CudaGpuMemSizeSum
    from nsys_recipe.cuda_gpu_mem_time_sum.cuda_gpu_mem_time_sum import CudaGpuMemTimeSum
    from nsys_recipe.nvtx_gpu_proj_trace.nvtx_gpu_proj_trace import NvtxGpuProjTrace
    from nsys_recipe.osrt_sum.osrt_sum import OsrtSum

    # Heatmap recipes
    from nsys_recipe.cuda_gpu_time_util_map.cuda_gpu_time_util_map import CudaGpuTimeUtilMap
    from nsys_recipe.gpu_metric_util_map.gpu_metric_util_map import GpuMetricUtilMap

    # Pandas recipes
    from nsys_recipe.cuda_gpu_kern_pace.cuda_gpu_kern_pace import CudaGpuKernPace
    from nsys_recipe.mpi_sum.mpi_sum import MpiSum
    from nsys_recipe.nccl_gpu_proj_sum.nccl_gpu_proj_sum import NcclGpuProjSum
    from nsys_recipe.nccl_sum.nccl_sum import NcclSum
    from nsys_recipe.nvtx_gpu_proj_pace.nvtx_gpu_proj_pace import NvtxGpuProjPace
    from nsys_recipe.nvtx_gpu_proj_sum.nvtx_gpu_proj_sum import NvtxGpuProjSum
    from nsys_recipe.nvtx_pace.nvtx_pace import NvtxPace
    from nsys_recipe.nvtx_sum.nvtx_sum import NvtxSum

except ModuleNotFoundError as e:
    req_file = nsys_constants.NSYS_RECIPE_REQ_COMMON_PATH
    install_file = nsys_constants.NSYS_RECIPE_INSTALL_PATH

    sys.exit(f"ERROR: {e}\nAll packages listed in '{req_file}' must be installed."
        f" You can automate the installation using the '{install_file}' script."
        " For more information, please refer to the Nsight Systems User Guide.")
